@@ -3,7 +3,6 @@ import io
 import sys
 import os
 import ujson
-import fido
 import time
 import gzip
 import warnings
@@ -35,8 +34,7 @@ from swagger_spec_validator.common import SwaggerValidationError
 from jsonschema.exceptions import ValidationError
 from bravado_core.formatter import SwaggerFormat
 from bravado.client import SwaggerClient
-from bravado.fido_client import FidoClient  # async
-from bravado.http_future import HttpFuture
+from bravado.requests_client import RequestsClient
 from bravado.swagger_model import Loader
 from bravado.config import bravado_config_from_config_dict
 from bravado_core.spec import Spec
@@ -46,7 +44,7 @@ from json2html import Json2Html
 from IPython.display import display, HTML, Image, FileLink
 from boltons.iterutils import remap
 from pymatgen.core import Structure as PmgStructure
-from concurrent.futures import as_completed
+from concurrent.futures import as_completed, ProcessPoolExecutor
 from requests_futures.sessions import FuturesSession
 from urllib3.util.retry import Retry
 from filetype.types.archive import Gz
@@ -108,6 +106,8 @@ ureg.define("electron_mass = 9.1093837015e-31 kg = mₑ = m_e")
 
 LOG_LEVEL = os.environ.get("MPCONTRIBS_CLIENT_LOG_LEVEL", "INFO")
 log_level = getattr(logging, LOG_LEVEL.upper())
+_session = requests.Session()
+_executor = ProcessPoolExecutor(max_workers=MAX_WORKERS)
 
 
 class LogFilter(logging.Filter):
@@ -208,7 +208,7 @@ def grouper(n, iterable):
         yield chunk
 
 
-def get_session(max_workers=MAX_WORKERS):
+def get_session(session=None):
     # TODO add Bad Gateway 502?
     adapter_kwargs = dict(max_retries=Retry(
         total=RETRIES,
@@ -217,9 +217,12 @@ def get_session(max_workers=MAX_WORKERS):
         respect_retry_after_header=True,
         status_forcelist=[429],  # rate limit
     ))
-    s = FuturesSession(max_workers=max_workers, adapter_kwargs=adapter_kwargs)
-    s.hooks['response'].append(_response_hook)
-    return s
+    s = session if session else _session
+    futures_session = FuturesSession(
+        session=s, executor=_executor, adapter_kwargs=adapter_kwargs
+    )
+    futures_session.hooks['response'].append(_response_hook)
+    return futures_session
 
 
 def _response_hook(resp, *args, **kwargs):
@@ -246,20 +249,6 @@ def _response_hook(resp, *args, **kwargs):
         resp.count = 0
 
 
-class FidoClientGlobalHeaders(FidoClient):
-    def __init__(self, headers=None):
-        super().__init__()
-        self.headers = headers or {}
-
-    def request(self, request_params, operation=None, request_config=None):
-        request_for_twisted = self.prepare_request_for_twisted(request_params)
-        request_for_twisted["headers"].update(self.headers)
-        future_adapter = self.future_adapter_class(fido.fetch(**request_for_twisted))
-        return HttpFuture(
-            future_adapter, self.response_adapter_class, operation, request_config
-        )
-
-
 def visit(path, key, value):
     if isinstance(value, dict) and "display" in value:
         return key, value["display"]
@@ -434,7 +423,8 @@ def _run_futures(futures, total: int = 0, timeout: int = -1, desc=None, disable=
 @functools.lru_cache(maxsize=1000)
 def _load(protocol, host, headers_json, project, version):
     headers = ujson.loads(headers_json)
-    http_client = FidoClientGlobalHeaders(headers=headers)
+    http_client = RequestsClient()
+    http_client.session.headers.update(headers)
     url = f"{protocol}://{host}"
     origin_url = f"{url}/apispec.json"
     url4fn = origin_url.replace("apispec", f"apispec-{version}").encode('utf-8')
@@ -476,15 +466,10 @@ def _load(protocol, host, headers_json, project, version):
         return swagger_spec
 
     # expand regex-based query parameters for `data` columns
-    session = get_session()
     query = {"name": project} if project else {}
     query["_fields"] = ["columns"]
     kwargs = dict(headers=headers, params=query)
-    future = session.get(f"{url}/projects/", **kwargs)
-    track_id = "get_columns"
-    setattr(future, "track_id", track_id)
-    resp = _run_futures([future], timeout=3, disable=True).get(track_id, {}).get("result")
-    session.close()
+    resp = _session.get(f"{url}/projects/", **kwargs).json()
 
     if not resp or not resp["data"]:
         raise ValueError(f"Failed to load projects for query {query}!")
@@ -526,6 +511,36 @@ def _load(protocol, host, headers_json, project, version):
     return swagger_spec
 
 
+@functools.lru_cache(maxsize=1)
+def _version(url):
+    retries, max_retries = 0, 3
+    protocol = urlparse(url).scheme
+    is_mock_test = 'pytest' in sys.modules and protocol == "http"
+
+    if is_mock_test:
+        now = datetime.datetime.now()
+        return Version(
+            major=now.year, minor=now.month, patch=now.day,
+            prerelease=(str(now.hour), str(now.minute))
+        )
+    else:
+        while retries < max_retries:
+            try:
+                r = requests.get(f"{url}/healthcheck", timeout=2)
+                if r.status_code == 200:
+                    return r.json().get("version")
+                else:
+                    retries += 1
+                    logger.warning(
+                        f"Healthcheck for {url} failed ({r.status_code})! Wait 30s."
+                    )
+                    time.sleep(30)
+            except RequestException as ex:
+                retries += 1
+                logger.warning(f"Could not connect to {url} ({ex})! Wait 30s.")
+                time.sleep(30)
+
+
 class Client(SwaggerClient):
     """client to connect to MPContribs API
 
@@ -535,16 +550,13 @@ class Client(SwaggerClient):
           >>> from mpcontribs.client import Client
           >>> client = Client()
     """
-    # Borg: https://www.oreilly.com/library/view/python-cookbook/0596001673/ch05s23.html
-    # NOTE bravado future doesn't work with concurrent.futures
-    _shared_state = {}
-
     def __init__(
         self,
         apikey: str = None,
         headers: dict = None,
         host: str = None,
-        project: str = None
+        project: str = None,
+        session: requests.Session = None,
     ):
         """Initialize the client - only reloads API spec from server as needed
 
@@ -553,11 +565,11 @@ class Client(SwaggerClient):
             headers (dict): custom headers for localhost connections
             host (str): host address to connect to (or use MPCONTRIBS_API_HOST env var)
             project (str): use this project for all operations (query, update, create, delete)
+            session (requests.Session): override session for client to use
         """
+        # NOTE bravado future doesn't work with concurrent.futures
         # - Kong forwards consumer headers when api-key used for auth
         # - forward consumer headers when connecting through localhost
-        self.__dict__ = self._shared_state
-
         if not host:
             host = os.environ.get("MPCONTRIBS_API_HOST", DEFAULT_HOST)
 
@@ -582,44 +594,15 @@ class Client(SwaggerClient):
         if self.url not in VALID_URLS:
             raise ValueError(f"{self.url} not a valid URL (one of {VALID_URLS})")
 
-        if "version" not in self.__dict__:
-            retries, max_retries = 0, 3
-            is_mock_test = 'pytest' in sys.modules and self.protocol == "http"
-
-            if is_mock_test:
-                now = datetime.datetime.now()
-                self.version = Version(
-                    major=now.year, minor=now.month, patch=now.day,
-                    prerelease=(str(now.hour), str(now.minute))
-                )
-            else:
-                while retries < max_retries:
-                    try:
-                        r = requests.get(f"{self.url}/healthcheck", timeout=2)
-                        if r.status_code == 200:
-                            self.version = r.json().get("version")
-                            break
-                        else:
-                            retries += 1
-                            logger.warning(
-                                f"Healthcheck for {self.url} failed ({r.status_code})! Wait 30s."
-                            )
-                            time.sleep(30)
-                    except RequestException as ex:
-                        retries += 1
-                        logger.warning(f"Could not connect to {self.url} ({ex})! Wait 30s.")
-                        time.sleep(30)
-
-        if "session" not in self.__dict__:
-            self.session = get_session()
-
+        self.version = _version(self.url)  # includes healthcheck
+        self.session = get_session(session=session)
         super().__init__(self.cached_swagger_spec)
 
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        return self.session.close()
+        return None
 
     @property
     def cached_swagger_spec(self):
@@ -655,13 +638,14 @@ class Client(SwaggerClient):
 
         return True, None
 
-    def _get_per_page_default_max(self, op: str = "get", resource: str = "contributions") -> int:
+    def _get_per_page_default_max(self, op: str = "query", resource: str = "contributions") -> int:
         resource = self.swagger_spec.resources[resource]
-        param_spec = getattr(resource, f"{op}_entries").params["per_page"].param_spec
+        attr = f"{op}{resource.capitalize()}"
+        param_spec = getattr(resource, attr).params["per_page"].param_spec
         return param_spec["default"], param_spec["maximum"]
 
     def _get_per_page(
-        self, per_page: int, op: str = "get", resource: str = "contributions"
+        self, per_page: int, op: str = "query", resource: str = "contributions"
     ) -> int:
         _, per_page_max = self._get_per_page_default_max(op=op, resource=resource)
         return min(per_page_max, per_page)
@@ -669,7 +653,7 @@ class Client(SwaggerClient):
     def _split_query(
         self,
         query: dict,
-        op: str = "get",
+        op: str = "query",
         resource: str = "contributions",
         pages: int = -1,
     ) -> List[dict]:
@@ -719,15 +703,13 @@ class Client(SwaggerClient):
         track_id,
         params: dict,
         rel_url: str = "contributions",
-        op: str = "get",
+        op: str = "query",
         data: dict = None
     ):
-        if self.session and self.session.executor._shutdown:
-            raise ValueError("Session closed. Use `with` statement.")
-
         rname = rel_url.split("/", 1)[0]
         resource = self.swagger_spec.resources[rname]
-        method = getattr(resource, f"{op}_entries").http_method
+        attr = f"{op}{resource.capitalize()}"
+        method = getattr(resource, attr).http_method
         kwargs = dict(headers=self.headers, params=params)
 
         if method == "put" and data:
@@ -749,7 +731,7 @@ class Client(SwaggerClient):
         if not name:
             return {"error": "initialize client with project or set `name` argument!"}
 
-        return Dict(self.projects.get_entry(pk=name, _fields=["_all"]).result())
+        return Dict(self.projects.getProjectByName(pk=name, _fields=["_all"]).result())
 
     def create_project(self, name: str, title: str, authors: str, description: str, url: str):
         """Create a project
@@ -771,7 +753,7 @@ class Client(SwaggerClient):
             "name": name, "title": title, "authors": authors, "description": description,
             "references": [{"label": "REF", "url": url}]
         }
-        owner = self.projects.create_entry(project=project).result().get("owner")
+        owner = self.projects.createProject(project=project).result().get("owner")
         logger.info(f"Project `{name}` created with owner `{owner}`")
 
     def get_contribution(self, cid: str) -> Type[Dict]:
@@ -782,7 +764,7 @@ class Client(SwaggerClient):
         """
         fields = list(self.get_model("ContributionsSchema")._properties.keys())
         fields.remove("needs_build")  # internal field
-        return Dict(self.contributions.get_entry(pk=cid, _fields=fields).result())
+        return Dict(self.contributions.getContributionById(pk=cid, _fields=fields).result())
 
     def get_table(self, tid_or_md5: str) -> Type[Table]:
         """Retrieve full Pandas DataFrame for a table
@@ -795,7 +777,7 @@ class Client(SwaggerClient):
             raise ValueError(f"'{tid_or_md5}' is not a valid table id or md5 hash digest!")
 
         if str_len == 32:
-            tables = self.tables.get_entries(md5=tid_or_md5, _fields=["id"]).result()
+            tables = self.tables.queryTables(md5=tid_or_md5, _fields=["id"]).result()
             if not tables:
                 raise ValueError(f"table for md5 '{tid_or_md5}' not found!")
             tid = tables["data"][0]["id"]
@@ -808,7 +790,7 @@ class Client(SwaggerClient):
         page, pages = 1, None
 
         while pages is None or page <= pages:
-            resp = self.tables.get_entry(
+            resp = self.tables.getTableById(
                 pk=tid, _fields=["_all"], data_page=page, data_per_page=per_page
             ).result()
             table["data"].extend(resp["data"])
@@ -849,7 +831,7 @@ class Client(SwaggerClient):
             raise ValueError(f"'{sid_or_md5}' is not a valid structure id or md5 hash digest!")
 
         if str_len == 32:
-            structures = self.structures.get_entries(md5=sid_or_md5, _fields=["id"]).result()
+            structures = self.structures.queryStructures(md5=sid_or_md5, _fields=["id"]).result()
             if not structures:
                 raise ValueError(f"structure for md5 '{sid_or_md5}' not found!")
             sid = structures["data"][0]["id"]
@@ -857,7 +839,7 @@ class Client(SwaggerClient):
             sid = sid_or_md5
 
         fields = list(self.get_model("StructuresSchema")._properties.keys())
-        resp = self.structures.get_entry(pk=sid, _fields=fields).result()
+        resp = self.structures.getStructureById(pk=sid, _fields=fields).result()
         ret = Structure.from_dict(resp)
         ret.attrs = {
             field: resp[field]
@@ -876,7 +858,7 @@ class Client(SwaggerClient):
             raise ValueError(f"'{aid_or_md5}' is not a valid attachment id or md5 hash digest!")
 
         if str_len == 32:
-            attachments = self.attachments.get_entries(
+            attachments = self.attachments.queryAttachments(
                 md5=aid_or_md5, _fields=["id"]
             ).result()
             if not attachments:
@@ -885,7 +867,7 @@ class Client(SwaggerClient):
         else:
             aid = aid_or_md5
 
-        return Attachment(self.attachments.get_entry(pk=aid, _fields=["_all"]).result())
+        return Attachment(self.attachments.getAttachmentById(pk=aid, _fields=["_all"]).result())
 
     def init_columns(self, columns: dict = None) -> dict:
         """initialize columns for a project to set their order and desired units
@@ -969,7 +951,7 @@ class Client(SwaggerClient):
             sorted_columns = flatten(unflatten(columns, splitter="dot"), reducer="dot")
 
             # reconcile with existing columns
-            resp = self.projects.get_entry(pk=self.project, _fields=["columns"]).result()
+            resp = self.projects.getProjectById(pk=self.project, _fields=["columns"]).result()
             existing_columns = {}
 
             for col in resp["columns"]:
@@ -1012,7 +994,7 @@ class Client(SwaggerClient):
         if not valid:
             return {"error": error}
 
-        return self.projects.update_entry(pk=self.project, project=payload).result()
+        return self.projects.updateProjectByName(pk=self.project, project=payload).result()
 
     def delete_contributions(self, query: dict = None, timeout: int = -1):
         """Remove all contributions for a query
@@ -1059,7 +1041,7 @@ class Client(SwaggerClient):
         query: dict = None,
         timeout: int = -1,
         resource: str = "contributions",
-        op: str = "get"
+        op: str = "query"
     ) -> tuple:
         """Retrieve total count and pages for resource entries matching query
 
@@ -1068,12 +1050,12 @@ class Client(SwaggerClient):
             timeout (int): cancel remaining requests if timeout exceeded (in seconds)
             resource (str): type of resource
             op (str): operation to calculate total pages for, one of
-                      ("get", "create", "update", "delete", "download")
+                      ("query", "create", "update", "delete", "download")
 
         Returns:
             tuple of total counts and pages
         """
-        ops = {"get", "create", "update", "delete", "download"}
+        ops = {"query", "create", "update", "delete", "download"}
         if op not in ops:
             logger.error(f"`op` has to be one of {ops}")
             return
@@ -1112,7 +1094,7 @@ class Client(SwaggerClient):
         elif self.project:
             query = {"name": self.project}
 
-        resp = self.projects.get_entries(
+        resp = self.projects.queryProjects(
             _fields=["name", "unique_identifiers"], **query
         ).result()
 
@@ -1129,7 +1111,7 @@ class Client(SwaggerClient):
         timeout: int = -1,
         data_id_fields: dict = None,
         fmt: str = "sets",
-        op: str = "get",
+        op: str = "query",
     ) -> dict:
         """Retrieve a list of existing contribution and component (Object)IDs
 
@@ -1140,7 +1122,7 @@ class Client(SwaggerClient):
             data_id_fields (dict): map of project to extra field in `data` to include as ID field
             fmt (str): return `sets` of identifiers or `map` (see below)
             op (str): operation to calculate total pages for, one of
-                      ("get", "create", "update", "delete", "download")
+                      ("query", "create", "update", "delete", "download")
 
         Returns:
             {"<project-name>": {
@@ -1188,7 +1170,7 @@ class Client(SwaggerClient):
             logger.error(f"`fmt` must be subset of {fmts}!")
             return
 
-        ops = {"get", "create", "update", "delete", "download"}
+        ops = {"query", "create", "update", "delete", "download"}
         if op not in ops:
             logger.error(f"`op` has to be one of {ops}")
             return
@@ -1292,7 +1274,7 @@ class Client(SwaggerClient):
     ) -> List[dict]:
         """Query contributions
 
-        See `client.contributions.get_entries()` for keyword arguments used in query.
+        See `client.contributions.queryContributions()` for keyword arguments used in query.
 
         Args:
             query (dict): optional query to select contributions
@@ -1333,7 +1315,7 @@ class Client(SwaggerClient):
                 ret["data"].extend(result["data"])
                 ret["total_count"] += result["total_count"]
         else:
-            ret = self.contributions.get_entries(
+            ret = self.contributions.queryContributions(
                 _fields=fields, _sort=sort, **query
             ).result()
 
@@ -1347,7 +1329,7 @@ class Client(SwaggerClient):
     ) -> dict:
         """Apply the same update to all contributions in a project (matching query)
 
-        See `client.contributions.get_entries()` for keyword arguments used in query.
+        See `client.contributions.queryContributions()` for keyword arguments used in query.
 
         Args:
             data (dict): update to apply on every matching contribution
@@ -1383,7 +1365,7 @@ class Client(SwaggerClient):
             return
 
         # get current list of data columns to decide if swagger reload is needed
-        resp = self.projects.get_entry(pk=self.project, _fields=["columns"]).result()
+        resp = self.projects.getProjectByName(pk=self.project, _fields=["columns"]).result()
         old_paths = set(c["path"] for c in resp["columns"])
 
         total = len(cids)
@@ -1398,7 +1380,7 @@ class Client(SwaggerClient):
         updated = sum(resp["count"] for _, resp in responses.items())
 
         if updated:
-            resp = self.projects.get_entry(pk=self.project, _fields=["columns"]).result()
+            resp = self.projects.getProjectByName(pk=self.project, _fields=["columns"]).result()
             new_paths = set(c["path"] for c in resp["columns"])
 
             if new_paths != old_paths:
@@ -1464,7 +1446,7 @@ class Client(SwaggerClient):
             query["project"] = self.project
 
         try:
-            resp = self.projects.get_entry(
+            resp = self.projects.getProjectByName(
                 pk=query["project"], _fields=["is_public", "is_approved"]
             ).result()
         except HTTPNotFound:
@@ -1479,7 +1461,7 @@ class Client(SwaggerClient):
             if is_public and not resp["is_approved"]:
                 return {"error": f"project `{query['project']}` is not approved yet!"}
 
-            resp = self.projects.update_entry(
+            resp = self.projects.updateProjectByName(
                 pk=query["project"], project={"is_public": is_public}
             ).result()
             ret["published"] = resp["is_public"] == is_public
@@ -1487,7 +1469,7 @@ class Client(SwaggerClient):
         if recursive:
             query = query or {}
             query["is_public"] = not is_public
-            ret["contributions"] = self.update_contributions(
+            ret["contributions"] = self.updateContributions(
                 {"is_public": is_public}, query=query, timeout=timeout
             )
 
@@ -1721,12 +1703,7 @@ class Client(SwaggerClient):
 
         # submit contributions
         if contribs:
-            if self.session and self.session.executor._shutdown:
-                raise ValueError("Session closed. Use `with` statement.")
-
             total, total_processed = 0, 0
-            self.session.close()
-            self.session = get_session(max_workers=4)
 
             def post_future(track_id, payload):
                 future = self.session.post(
@@ -1796,7 +1773,7 @@ class Client(SwaggerClient):
                         existing[project_name] = self.get_all_ids(
                             dict(project=project_name), include=COMPONENTS, timeout=timeout
                         ).get(project_name, {"identifiers": set()})
-                        unique_identifiers[project_name] = self.projects.get_entry(
+                        unique_identifiers[project_name] = self.projects.getProjectByName(
                             pk=project_name, _fields=["unique_identifiers"]
                         ).result()["unique_identifiers"]
                         existing_ids = existing.get(project_name, {}).get("identifiers", [])
@@ -1819,8 +1796,6 @@ class Client(SwaggerClient):
             dt = (toc - tic) / 60
             self.init_columns()
             self._reinit()
-            self.session.close()
-            self.session = get_session()
             logger.info(f"It took {dt:.1f}min to submit {total_processed}/{total} contributions.")
         else:
             logger.info("Nothing to submit.")
